<?php
/**
 * Plugin Name: Auto Featured Image
 * Plugin URI: https://teesharp.com/auto-featured-image
 * Description: Automatically sets the first image in a post as the featured image
 * Version: 1.0.0
 * Author: Teesharp Limited
 * Author URI: https://teesharp.com
 * Text Domain: auto-featured-image
 * License: GPL v2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.txt
 */

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit;
}

class Auto_Featured_Image {
    
    /**
     * Constructor
     */
    public function __construct() {
        // Hook into WordPress save_post action
        add_action('save_post', array($this, 'set_featured_image'), 10, 3);
        
        // Add settings page
        add_action('admin_menu', array($this, 'add_settings_page'));
        add_action('admin_init', array($this, 'register_settings'));
        
        // Add meta box to enable/disable per post
        add_action('add_meta_boxes', array($this, 'add_meta_box'));
        add_action('save_post', array($this, 'save_meta_box_data'));
    }
    
    /**
     * Set featured image when post is saved
     */
    public function set_featured_image($post_id, $post, $update) {
        // Skip if autosave
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }
        
        // Skip if not a post or page (based on settings)
        $post_types = $this->get_enabled_post_types();
        if (!in_array($post->post_type, $post_types)) {
            return;
        }
        
        // Skip if featured image already exists
        if (has_post_thumbnail($post_id) && !get_option('afi_override_existing', false)) {
            return;
        }
        
        // Skip if specifically disabled for this post
        if (get_post_meta($post_id, '_afi_disable', true) === '1') {
            return;
        }
        
        // Skip if post status is not published or scheduled
        if (!in_array($post->post_status, array('publish', 'future'))) {
            return;
        }
        
        // Get post content
        $content = $post->post_content;
        
        // Find the first image in the content
        if (preg_match('/<img.+?src=[\'"]([^\'"]+)[\'"].*?>/i', $content, $matches)) {
            $image_url = $matches[1];
            
            // Check if image URL is valid
            if (empty($image_url)) {
                return;
            }
            
            // Process URL to handle relative paths
            if (strpos($image_url, 'http') !== 0) {
                if (strpos($image_url, '//') === 0) {
                    $image_url = 'https:' . $image_url;
                } else {
                    $image_url = site_url($image_url);
                }
            }
            
            // Check if the image is already in the media library
            $attachment_id = $this->get_attachment_id_from_url($image_url);
            
            if (!$attachment_id) {
                // If using external images is enabled
                if (get_option('afi_external_images', true)) {
                    // Download and attach the image
                    $attachment_id = $this->download_and_attach_image($image_url, $post_id);
                }
            }
            
            if ($attachment_id) {
                // Set as featured image
                set_post_thumbnail($post_id, $attachment_id);
            }
        }
    }
    
    /**
     * Get attachment ID from URL
     */
    private function get_attachment_id_from_url($url) {
        global $wpdb;
        
        // Remove query string
        $url = preg_replace('/\?.*/', '', $url);
        
        // Get the attachment ID from the database
        $attachment = $wpdb->get_col($wpdb->prepare("SELECT ID FROM $wpdb->posts WHERE guid='%s';", $url));
        
        // If not found, try with site URL removed (handles domain changes)
        if (empty($attachment)) {
            $site_url = site_url();
            $url_no_domain = str_replace($site_url, '', $url);
            
            if ($url_no_domain !== $url) {
                $attachment = $wpdb->get_col($wpdb->prepare("SELECT ID FROM $wpdb->posts WHERE guid LIKE '%%%s';", $url_no_domain));
            }
        }
        
        return isset($attachment[0]) ? $attachment[0] : false;
    }
    
    /**
     * Download and attach external image
     */
    private function download_and_attach_image($url, $post_id) {
        // Skip if URL is not valid
        if (!filter_var($url, FILTER_VALIDATE_URL)) {
            return false;
        }
        
        // Get WordPress upload directory
        $upload_dir = wp_upload_dir();
        
        // Get file name
        $file_name = basename($url);
        
        // Remove query string if present
        $file_name = preg_replace('/\?.*/', '', $file_name);
        
        // Generate unique filename
        $unique_file_name = wp_unique_filename($upload_dir['path'], $file_name);
        $file_path = $upload_dir['path'] . '/' . $unique_file_name;
        
        // Download file to temporary location
        $temp_file = download_url($url);
        
        // Check for download errors
        if (is_wp_error($temp_file)) {
            return false;
        }
        
        // Move temp file to uploads directory
        $moved = copy($temp_file, $file_path);
        @unlink($temp_file); // Clean up temp file
        
        if (!$moved) {
            return false;
        }
        
        // Get file type
        $file_type = wp_check_filetype($unique_file_name, null);
        
        // Prepare attachment data
        $attachment = array(
            'guid'           => $upload_dir['url'] . '/' . $unique_file_name,
            'post_mime_type' => $file_type['type'],
            'post_title'     => preg_replace('/\.[^.]+$/', '', $unique_file_name),
            'post_content'   => '',
            'post_status'    => 'inherit'
        );
        
        // Insert attachment
        $attachment_id = wp_insert_attachment($attachment, $file_path, $post_id);
        
        if (!is_wp_error($attachment_id)) {
            // Include image functions
            require_once(ABSPATH . 'wp-admin/includes/image.php');
            
            // Generate metadata
            $attachment_data = wp_generate_attachment_metadata($attachment_id, $file_path);
            wp_update_attachment_metadata($attachment_id, $attachment_data);
            
            return $attachment_id;
        }
        
        return false;
    }
    
    /**
     * Get enabled post types
     */
    private function get_enabled_post_types() {
        $post_types = get_option('afi_post_types', array('post'));
        
        if (!is_array($post_types)) {
            $post_types = array('post');
        }
        
        return $post_types;
    }
    
    /**
     * Add settings page
     */
    public function add_settings_page() {
        add_options_page(
            __('Auto Featured Image Settings', 'auto-featured-image'),
            __('Auto Featured Image', 'auto-featured-image'),
            'manage_options',
            'auto-featured-image',
            array($this, 'render_settings_page')
        );
    }
    
    /**
     * Register settings
     */
    public function register_settings() {
        register_setting('auto_featured_image_settings', 'afi_post_types');
        register_setting('auto_featured_image_settings', 'afi_override_existing', array(
            'type' => 'boolean',
            'default' => false
        ));
        register_setting('auto_featured_image_settings', 'afi_external_images', array(
            'type' => 'boolean',
            'default' => true
        ));
    }
    
    /**
     * Render settings page
     */
    public function render_settings_page() {
        $post_types = get_post_types(array('public' => true), 'objects');
        $enabled_post_types = $this->get_enabled_post_types();
        ?>
        <div class="wrap">
            <h1><?php _e('Auto Featured Image Settings', 'auto-featured-image'); ?></h1>
            
            <form method="post" action="options.php">
                <?php settings_fields('auto_featured_image_settings'); ?>
                <?php do_settings_sections('auto_featured_image_settings'); ?>
                
                <table class="form-table">
                    <tr valign="top">
                        <th scope="row"><?php _e('Post Types', 'auto-featured-image'); ?></th>
                        <td>
                            <?php foreach ($post_types as $post_type) : ?>
                                <label>
                                    <input type="checkbox" name="afi_post_types[]" 
                                           value="<?php echo esc_attr($post_type->name); ?>" 
                                           <?php checked(in_array($post_type->name, $enabled_post_types)); ?>>
                                    <?php echo esc_html($post_type->label); ?>
                                </label><br>
                            <?php endforeach; ?>
                            <p class="description">
                                <?php _e('Select which post types should have auto featured images.', 'auto-featured-image'); ?>
                            </p>
                        </td>
                    </tr>
                    
                    <tr valign="top">
                        <th scope="row"><?php _e('Override Existing', 'auto-featured-image'); ?></th>
                        <td>
                            <label>
                                <input type="checkbox" name="afi_override_existing" value="1" 
                                       <?php checked(get_option('afi_override_existing')); ?>>
                                <?php _e('Replace existing featured images', 'auto-featured-image'); ?>
                            </label>
                            <p class="description">
                                <?php _e('If enabled, the plugin will replace any existing featured image with the first image in the content.', 'auto-featured-image'); ?>
                            </p>
                        </td>
                    </tr>
                    
                    <tr valign="top">
                        <th scope="row"><?php _e('External Images', 'auto-featured-image'); ?></th>
                        <td>
                            <label>
                                <input type="checkbox" name="afi_external_images" value="1" 
                                       <?php checked(get_option('afi_external_images', true)); ?>>
                                <?php _e('Download external images', 'auto-featured-image'); ?>
                            </label>
                            <p class="description">
                                <?php _e('If enabled, the plugin will download external images to your media library.', 'auto-featured-image'); ?>
                            </p>
                        </td>
                    </tr>
                </table>
                
                <?php submit_button(); ?>
            </form>
        </div>
        <?php
    }
    
    /**
     * Add meta box to enable/disable per post
     */
    public function add_meta_box() {
        $post_types = $this->get_enabled_post_types();
        
        foreach ($post_types as $post_type) {
            add_meta_box(
                'afi_meta_box',
                __('Auto Featured Image', 'auto-featured-image'),
                array($this, 'render_meta_box'),
                $post_type,
                'side',
                'default'
            );
        }
    }
    
    /**
     * Render meta box
     */
    public function render_meta_box($post) {
        // Add nonce for security
        wp_nonce_field('afi_meta_box', 'afi_meta_box_nonce');
        
        // Get current value
        $disable = get_post_meta($post->ID, '_afi_disable', true);
        ?>
        <label>
            <input type="checkbox" name="afi_disable" value="1" <?php checked($disable, '1'); ?>>
            <?php _e('Disable auto featured image for this post', 'auto-featured-image'); ?>
        </label>
        <?php
    }
    
    /**
     * Save meta box data
     */
    public function save_meta_box_data($post_id) {
        // Check if nonce is valid
        if (!isset($_POST['afi_meta_box_nonce']) || !wp_verify_nonce($_POST['afi_meta_box_nonce'], 'afi_meta_box')) {
            return;
        }
        
        // Check if user has permissions
        if (!current_user_can('edit_post', $post_id)) {
            return;
        }
        
        // Check if autosave
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }
        
        // Save the data
        if (isset($_POST['afi_disable'])) {
            update_post_meta($post_id, '_afi_disable', '1');
        } else {
            delete_post_meta($post_id, '_afi_disable');
        }
    }
    
    /**
     * Plugin activation
     */
    public static function activate() {
        // Set default options
        add_option('afi_post_types', array('post'));
        add_option('afi_override_existing', false);
        add_option('afi_external_images', true);
    }
}

// Initialize the plugin
$auto_featured_image = new Auto_Featured_Image();

// Register activation hook
register_activation_hook(__FILE__, array('Auto_Featured_Image', 'activate'));